@@ -42,22 +42,30 @@ void RarTime::GetLocal(RarLocalTime *lt)
   GetWin32(&ft);
   FILETIME lft;
 
-  // We use these functions instead of FileTimeToLocalFileTime according to
-  // MSDN recommendation: "To account for daylight saving time
-  // when converting a file time to a local time ..."
-  SYSTEMTIME st1,st2;
-  FileTimeToSystemTime(&ft,&st1);
-  SystemTimeToTzSpecificLocalTime(NULL,&st1,&st2);
-  SystemTimeToFileTime(&st2,&lft);
+  if (WinNT() < WNT_VISTA)
+  {
+    // SystemTimeToTzSpecificLocalTime based code produces 1 hour error on XP.
+    FileTimeToLocalFileTime(&ft,&lft);
+  }
+  else
+  {
+    // We use these functions instead of FileTimeToLocalFileTime according to
+    // MSDN recommendation: "To account for daylight saving time
+    // when converting a file time to a local time ..."
+    SYSTEMTIME st1,st2;
+    FileTimeToSystemTime(&ft,&st1);
+    SystemTimeToTzSpecificLocalTime(NULL,&st1,&st2);
+    SystemTimeToFileTime(&st2,&lft);
 
-  // Correct precision loss (low 4 decimal digits) in FileTimeToSystemTime.
-  FILETIME rft;
-  SystemTimeToFileTime(&st1,&rft);
-  int64 Corrected=INT32TO64(ft.dwHighDateTime,ft.dwLowDateTime)-
-                  INT32TO64(rft.dwHighDateTime,rft.dwLowDateTime)+
-                  INT32TO64(lft.dwHighDateTime,lft.dwLowDateTime);
-  lft.dwLowDateTime=(DWORD)Corrected;
-  lft.dwHighDateTime=(DWORD)(Corrected>>32);
+    // Correct precision loss (low 4 decimal digits) in FileTimeToSystemTime.
+    FILETIME rft;
+    SystemTimeToFileTime(&st1,&rft);
+    int64 Corrected=INT32TO64(ft.dwHighDateTime,ft.dwLowDateTime)-
+                    INT32TO64(rft.dwHighDateTime,rft.dwLowDateTime)+
+                    INT32TO64(lft.dwHighDateTime,lft.dwLowDateTime);
+    lft.dwLowDateTime=(DWORD)Corrected;
+    lft.dwHighDateTime=(DWORD)(Corrected>>32);
+  }
 
   SYSTEMTIME st;
   FileTimeToSystemTime(&lft,&st);
@@ -121,21 +129,30 @@ void RarTime::SetLocal(RarLocalTime *lt)
     if (lft.dwLowDateTime<lt->Reminder)
       lft.dwHighDateTime++;
 
-    // Reverse procedure which we do in GetLocal.
-    SYSTEMTIME st1,st2;
-    FileTimeToSystemTime(&lft,&st2);
-    TzSpecificLocalTimeToSystemTime(NULL,&st2,&st1);
     FILETIME ft;
-    SystemTimeToFileTime(&st1,&ft);
 
-    // Correct precision loss (low 4 decimal digits) in FileTimeToSystemTime.
-    FILETIME rft;
-    SystemTimeToFileTime(&st2,&rft);
-    int64 Corrected=INT32TO64(lft.dwHighDateTime,lft.dwLowDateTime)-
-                    INT32TO64(rft.dwHighDateTime,rft.dwLowDateTime)+
-                    INT32TO64(ft.dwHighDateTime,ft.dwLowDateTime);
-    ft.dwLowDateTime=(DWORD)Corrected;
-    ft.dwHighDateTime=(DWORD)(Corrected>>32);
+    if (WinNT() < WNT_VISTA)
+    {
+      // TzSpecificLocalTimeToSystemTime based code produces 1 hour error on XP.
+      LocalFileTimeToFileTime(&lft,&ft);
+    }
+    else
+    {
+      // Reverse procedure which we do in GetLocal.
+      SYSTEMTIME st1,st2;
+      FileTimeToSystemTime(&lft,&st2);
+      TzSpecificLocalTimeToSystemTime(NULL,&st2,&st1);
+      SystemTimeToFileTime(&st1,&ft);
+
+      // Correct precision loss (low 4 decimal digits) in FileTimeToSystemTime.
+      FILETIME rft;
+      SystemTimeToFileTime(&st2,&rft);
+      int64 Corrected=INT32TO64(lft.dwHighDateTime,lft.dwLowDateTime)-
+                      INT32TO64(rft.dwHighDateTime,rft.dwLowDateTime)+
+                      INT32TO64(ft.dwHighDateTime,ft.dwLowDateTime);
+      ft.dwLowDateTime=(DWORD)Corrected;
+      ft.dwHighDateTime=(DWORD)(Corrected>>32);
+    }
 
     *this=ft;
   }
